<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */

namespace Concrete\Package\GntVoteList\Attribute\GntBallotbox;
defined('C5_EXECUTE') or die('Access Denied.');

use Page;
use Log;
use Concrete\Core\Search\ItemList\Database\AttributedItemList;
use Concrete\Core\Attribute\Controller as AttributeTypeController;
use Concrete\Core\Attribute\Key\Category as AttributeKeyCategory;
use Concrete\Core\Foundation\Object;
use Database;

class Controller extends AttributeTypeController
{

    const TOTAL_VOTES = 'total_votes';
    const SCORE = 'score';
    const DOWN_VOTES = 'down_votes';
    const UP_VOTES = 'up_votes';

    protected $searchIndexFieldDefinition = array (
        self::UP_VOTES    => array( 'type' => 'integer', 'options' => array( 'default' => null, 'notnull' => false )  ),
        self::DOWN_VOTES  => array( 'type' => 'integer', 'options' => array( 'default' => null, 'notnull' => false )  ),
        self::TOTAL_VOTES => array( 'type' => 'integer', 'options' => array( 'default' => null, 'notnull' => false )  ),
        self::SCORE       => array( 'type' => 'integer', 'options' => array( 'default' => null, 'notnull' => false )  ),
    );

    static public function getSearchDbField($akHandle, $field)
    {
        return "ak_${akHandle}_${field}";
    }

    public function searchDBField($field)
    {
        $akHandle = $this->getAttributeKeyHandle();
        return self::getSearchDbField($akHandle, $field);
    }

    public function getSearchIndexValue()
    {
        $v = $this->getValue();
        if ($v) {
            $up = $v->upVotes;
            $down = $v->downVotes;
            $args = array();
            $args[self::UP_VOTES]    = $up;
            $args[self::DOWN_VOTES]  = $down;
            $args[self::TOTAL_VOTES] = $up + $down;
            $args[self::SCORE]       = $up - $down;
            return $args;
        }
        return null;
    }

    public function getAttributeKeyHandle()
    {
        return $this->attributeKey->getAttributeKeyHandle();
    }

    public function getAttributeKeyCategoryHandle()
    {
        $id = $this->attributeKey->getAttributeKeyCategoryID();
        return AttributeKeyCategory::getById($id)->getAttributeKeyCategoryHandle();
    }

    //
    // Since this attribute has multiple values, we search by default onto one,
    // or allow the value to be specified in the old array(key=>value) fashion
    //
    public function filterByAttribute(AttributedItemList $list, $value, $comparison = '=')
    {
        $valueName = self::SCORE;
        $val = $value;

        if (is_array($value)) {
            $valueName = key($value);
            $val = current($value);
        }

        ob_start();
        var_dump($val);
        $valstr = ob_get_clean();
        ob_start();
        var_dump($value);
        $valuestr = ob_get_clean();

        $list->filter($this->searchDBField($valueName), $val, $comparison);
    }


    public function getValue()
    {
        $val = Value::getByID($this->getAttributeValueID());
        return $val;
    }

    public function getDisplayValue()
    {
        return nl2br($this->getValue());
    }

    public function form()
    {
        // Check that we have a current value to use, and use it
        if (is_object ($this->attributeValue)) {
            $val = $this->getValue();
            $this->set('upVotes', $val->upVotes);
            $this->set('downVotes', $val->downVotes);
        }
        // form.php will be rendered after this point
    }

    public function type_form() // used to edit the type itself (in the page attribute form)
    {
        // type_form.php will be rendered after this point
    }

    public function saveValue($data)
    {
        if ($data instanceof Value) {
            $data = (array) $data;
        }
        extract($data);

        Database::getActiveConnection()->Replace(
            'atGntBallotBox'
            , array('avID' => $this->getAttributeValueID(),
                    'upVotes' => intval($upVotes),
                    'downVotes' => intval($downVotes),
            ),
            'avID', true
        );
    }

    public function saveForm($data)
    {
        $this->saveValue($data);
    }

    public function validateForm($data)
    {
        if (!isset( $data['upVotes'], $data['downVotes'])) {
            return false;
        }

        if (!is_numeric($data['upVotes'], $data['downVotes'])) {
            return false;
        }
        return true;
    }

    public function deleteKey()
    {
        $db = Database::getActiveConnection();
        $arr = $this->attributeKey->getAttributeValueIDList();
        foreach($arr as $id){
            $db->Execute('delete from atGntBallotBox where avID = ?', array($id));
        }
    }

    public function deleteValue()
    {
        $db = Database::getActiveConnection();
        $db->Execute('delete from atGntBallotBox where avID = ?', array($this->getAttributeValueID()));
    }

    public static function newValue()
    {
        return new Value();
    }
}

class Value extends Object
{
    public $upVotes = 0;
    public $downVotes = 0;

    public function setPropertiesFromArray($values)
    {
        parent::setPropertiesFromArray($values);
        $this->upVotes = intval($this->upVotes);
        $this->downVotes = intval($this->downVotes);
    }

    public function getByID($avID)
    {
        $db = Database::getActiveConnection();
        $values = $db->GetRow("select avID, upVotes, downVotes from atGntBallotBox where avID = ?", array($avID));
        if (!isset( $values['avID'])) {
            return null;
        }

        $av = new self();
        $av->setPropertiesFromArray($values);
        return $av;
    }

    public function setUpVotes($v)
    {
        $this->upVotes = intval($v);
    }

    public function setDownVotes($v)
    {
        $this->downVotes = intval($v);
    }

    public function getUpVotes()
    {
        return intval($this->upVotes);
    }

    public function getDownVotes()
    {
        return intval($this->downVotes);
    }

    public function addUpVotes($v)
    {
        $this->upVotes += intval($v);
    }

    public function addDownVotes($v)
    {
        $this->downVotes += intval($v);
    }

    public function getTotalVotes()
    {
        return $this->upVotes + $this->downVotes;
    }

    public function getScore()
    {
        return $this->upVotes - $this->downVotes;
    }

    public function __toString()
    {
        return t('Up votes: %s; Down votes: %s', $this->upVotes, $this->downVotes);
    }
}

// vim: set expandtab cindent ts=4 smarttab ai:
