<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Block\GntBallotbox;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Block\BlockController;
use Page;
use Concrete\Package\GntVoteList\Src\PageBallotBox;
use Concrete\Core\Block\View\BlockViewTemplate;
use Concrete\Package\GntVoteList\Src\BallotBox;

class Controller extends BlockController
{

    protected $btTable = "btGntBallotBox";
    protected $btInterfaceWidth = "540";
    protected $btInterfaceHeight = "505";
    protected $btCacheBlockOutput = false;
    protected $btCacheBlockOutputOnPost = false;
    protected $btCacheBlockOutputForRegisteredUsers = false;
    protected $btWrapperClass = 'ccm-ui';

    protected $btDefaultSet = 'gnt_vote';

    private $_ballotBox = null;

    public function getBlockTypeDescription()
    {
        return t("Allow Users to Vote (up or down) for a page and display the result");
    }

    public function getBlockTypeName()
    {
        return t("Ballot Box");
    }

    public function save($args)
    {
        $args = $this->getBallotBox()->prepareSaveArray($args);
        $args['redirectNode'] = intval($args['redirectNode']);

        parent::save($args);
    }

    public function getBallotBox()
    {
        if (is_null($this->_ballotBox)) {
            $this->_ballotBox = new PageBallotBox(null, $this->akHandle);
            $this->_ballotBox->setController($this);
            $this->_ballotBox->install();
            $box = $this->_ballotBox;
        }

        return $this->_ballotBox;
    }

    public function add()
    {
        $this->edit();
    }

    public function edit()
    {
        $this->set('bbox', $this->getBallotBox());
    }

    public function view()
    {
        $uhs = \Core::make('helper/concrete/urls');

        $this->requireAsset('gnt_ballotbox');

        $box = $this->getBallotBox();

        $message = $box->computeMessage($this->boxComment);

        $this->set('canVote',   $box->canVote());
        $this->set('lastVote',  $box->getUserVote());
        $this->set('upVotes',   $up = $box->getUpVotes());
        $this->set('downVotes', $down = $box->getDownVotes());
        $this->set('ballotBox', $box);
        $this->set('targetCID', $box->getTargetCID());
        $this->set('boxComment', $message);
        $this->set('redirectLink', $box->getRedirectLink());

        $url = $uhs->getToolsURL('render_block', 'gnt_vote_list');

        $images = $box->getImagesArray();

        $this->set('onState', $onState = $box->canVote() ? BallotBox::STATE_ENABLED : BallotBox::STATE_DISABLED);
        $this->set('imageUp', $images[BallotBox::IMAGE_UP][$onState][$up ? BallotBox::STATE_FULL : BallotBox::STATE_EMPTY]);
        $this->set('imageDown', $images[BallotBox::IMAGE_DOWN][$onState][$down ? BallotBox::STATE_FULL : BallotBox::STATE_EMPTY]);

        $this->set('images', $images);
        $this->set('refreshLink', $url);
    }

    public function action_json_vote($bID = false)
    {
        if ($this->bID != $bID) {
            return false;
        }

        $cID = $this->get('targetCID');
        if ('' == $cID) {
            print json_encode(array( 'error' => t('invalid page selection') ));
            exit;
        }

        $akHandle = $this->akHandle;
        $box = new PageBallotBox($cID, $akHandle);
        $box->setController($this);
        print $box->jsONVote($this->get('vote'));
        exit;
    }

    public function getTemplatePath()
    {
        $block = $this->getBlockObject();

        if (!$block) {
            return DIR_PACKAGES.'/gnt_vote_list/blocks/gnt_ballotbox/view.php';
        }

        $bvt = new BlockViewTemplate($block);
        $filename = $block->getBlockFilename();
        if ($filename) {
            $bvt->setBlockCustomRender($filename);
        }

        $template = $bvt->getTemplate();

        return $template;
    }
}

