<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Block\GntVoteList;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Block\BlockController;
use Concrete\Core\Package\PackageList;
use Concrete\Core\Attribute\Key\CollectionKey as CollectionAttributeKey;
use Concrete\Package\GntVoteList\Src\BallotBox;
use Concrete\Package\GntVoteList\Src\PageBallotBox;
use Page;
use PageList;
use Database;
use Package;
use Log;
use Block;
use Concrete\Package\GntVoteList\Attribute\GntBallotbox\Controller as BallotBoxAttr;

class Controller extends BlockController
{

    protected $btTable = "btGntVoteList";
    protected $btInterfaceWidth = "730";
    protected $btInterfaceHeight = "480";
    protected $btCacheBlockOutput = false;
    protected $btCacheBlockOutputOnPost = false;
    protected $btCacheBlockOutputForRegisteredUsers = false;
    protected $btWrapperClass = 'ccm-ui';

    protected $btDefaultSet                        = 'gnt_vote';

    private $_templates = null;

    public function getBlockTypeDescription()
    {
        return t("List pages and sort them accordingly to their Up/Down vote count");
    }

    public function getBlockTypeName()
    {
        return t("Vote Page List");
    }

    # search types :
    const SEARCH_ALL        = 0;
    const SEARCH_CHILDS     = 1;
    const SEARCH_UNDER      = 2;
    const SEARCH_ALL_CHILDS = 3;
    const SEARCH_ALL_UNDER  = 4;

    public static function getSearchTypes()
    {
        return array(
            self::SEARCH_ALL        => t('Search all site pages'),
            self::SEARCH_CHILDS     => t('Pages directly under this page'),
            self::SEARCH_ALL_CHILDS => t('All pages under this page'),
            self::SEARCH_UNDER      => t('Pages directly under another page (to be selected)'),
            self::SEARCH_ALL_UNDER  => t('All Pages under another page (to be selected)'),
        );
    }

    const SORT_BY_SCORE                 = 0;
    const SORT_BY_TOTAL_VOTES           = 1;
    const SORT_BY_UP_VOTES              = 2;
    const SORT_BY_DOWN_VOTES            = 3;
    const SORT_BY_SITE_ORDER            = 4;
    const SORT_BY_PUBLIC_DATE           = 5;
    const SORT_BY_NAME                  = 6;
    const SORT_BY_TIMEFRAME_UP_VOTES    = 7;
    const SORT_BY_TIMEFRAME_DOWN_VOTES  = 8;
    const SORT_BY_TIMEFRAME_TOTAL_VOTES = 9;

    public static function getSortByTypes()
    {
        return array(
            self::SORT_BY_SCORE                 => t('up/down score'),
            self::SORT_BY_TOTAL_VOTES           => t('up/down total votes'),
            self::SORT_BY_UP_VOTES              => t('number of up votes'),
            self::SORT_BY_DOWN_VOTES            => t('number of down votes'),
            self::SORT_BY_TIMEFRAME_UP_VOTES    => t('number of up votes, within a timeframe'),
            self::SORT_BY_TIMEFRAME_DOWN_VOTES  => t('number of down votes, within a timeframe'),
            self::SORT_BY_TIMEFRAME_TOTAL_VOTES => t('number of total votes, within a timeframe'),
            self::SORT_BY_PUBLIC_DATE            => t('public date'),
            self::SORT_BY_NAME                    => t('page name'),
        );
    }

    const TIMEFRAME_SIZE_5MINUTES    = 0;
    const TIMEFRAME_SIZE_1HOUR       = 1;
    const TIMEFRAME_SIZE_1DAY        = 2;

    public static function getTimeFrameSizes()
    {
        return array(
            self::TIMEFRAME_SIZE_5MINUTES => t('5 minutes'),
            self::TIMEFRAME_SIZE_1HOUR    => t('1 hour'),
            self::TIMEFRAME_SIZE_1DAY     => t('1 day'),
        );
    }
    public static function sizeToInterval($size)
    {
        // SQL: DO NOT TRANSLATE THAT with t() !!!
        static $intervals = array(
            self::TIMEFRAME_SIZE_5MINUTES => '5 MINUTE',
            self::TIMEFRAME_SIZE_1HOUR    => '1 HOUR',
            self::TIMEFRAME_SIZE_1DAY     => '1 DAY',
        );

        return $intervals[intval($size)];
    }
    public static function sizeToBaseTs($size)
    {
        static $intervals = array(
            self::TIMEFRAME_SIZE_5MINUTES => 'DATE_FORMAT(DATE_FORMAT(ts,"%Y-%m-%d %H:00:00")+INTERVAL FLOOR(MINUTE(ts)/5)*5 MINUTE,"%Y-%m-%d %T")',
            self::TIMEFRAME_SIZE_1HOUR    => 'DATE_FORMAT(ts,"%Y-%m-%d %H:00:00")',
            self::TIMEFRAME_SIZE_1DAY     => 'DATE_FORMAT(ts,"%Y-%m-%d 00:00:00")',
        );

        return $intervals[intval($size)];
    }

    const TIMEFRAME_ENDS_NOW              = 0;
    const TIMEFRAME_ENDS_AT_LAST_RELEVANT = 1;

    public static function getTimeFrameEnds()
    {
        return array(
            self::TIMEFRAME_ENDS_NOW              => t('now'),
            self::TIMEFRAME_ENDS_AT_LAST_RELEVANT => t('at last relevant timeframe'),
        );
    }

    const SORT_TYPE_ASC = 0;
    const SORT_TYPE_DSC = 1;

    public static function getSortTypes()
    {
        return array(
            self::SORT_TYPE_ASC => t('Ascending (lower first)'),
            self::SORT_TYPE_DSC => t('Descending (highest first)'),
        );
    }

    const KEYWORD_NONE = 0;
    const KEYWORD_FIXED = 1;
    const KEYWORD_REQUEST = 2;

    public static function keywordSourceTypes()
    {
        return array(
            self::KEYWORD_NONE => t('No keyword search'),
            self::KEYWORD_FIXED => t('Fixed keyword search'),
            self::KEYWORD_REQUEST => t('Read keyword from a _REQUEST variable'),
        );
    }
    
    public function getPageList()
    {
        $list = new PageList();
        // $list->setNameSpace('btVoteList'.$this->bID); // XXX this had been removed? O_o?

        $akHandle = $this->akHandle;
        if ($this->filterOnlyVotables != 0) {
            $list->filterByAttribute($akHandle, array(BallotBoxAttr::TOTAL_VOTES => null), 'IS NOT');
        }

        if (self::isTimeFrameSort($this->sortBy)) {
            $list->filter('timeframe_total_votes', null, 'IS NOT');
        }

        $nbItemsPerPage = intval($this->nbItemsPerPage);
        if ($nbItemsPerPage > 0) {
            $list->setItemsPerPage($nbItemsPerPage);
        }

        $this->_applyFilters($list);

        if (false === $this->_applyKeywordSearch($list)) {
            return null;
        }

        $this->_applySort($list);

        return $list;
    }

    protected function _applyKeywordSearch(PageList $list)
    {
        $keyword = '';
        $varName = 'query';

        switch ($this->searchKeywordSource) {

            case self::KEYWORD_NONE:
                return true;

            case self::KEYWORD_FIXED:
                $keyword = $this->fixedKeyword;
                break;

            case self::KEYWORD_REQUEST:
                $varName = $this->keywordVariable;
                $keyword = '';
                if ($varName && isset($_REQUEST[$varName])) {
                    $keyword = trim($_REQUEST[$varName]);
                }
                
                if (!$keyword && !$this->allowEmptySearch) { 
                    // must kill the results at this point
                    return false;
                }
                break;
        }

        $keyword = trim($keyword);

        if ($keyword) {
            $list->filterByKeywords($keyword);
        }

        $this->set($varName, $keywords);
        return true;
    }

    protected function _applyFilters(PageList $list)
    {
        $akHandle = $this->akHandle;
        foreach (self::getFilterFields() as $filter => $field) {
            $minName = "${filter}Min";
            $maxName = "${filter}Max";
            $minEnableName = "${minName}Enabled";
            $maxEnableName = "${maxName}Enabled";
            $min = $this->$minName;
            $max = $this->$maxName;

            if ($this->$minEnableName) {
                $list->filterByAttribute($akHandle, array($field=>$min), '>=');
            }

            if ($this->$maxEnableName) {
                $list->filterByAttribute($akHandle, array($field=>$max), '<=');
            }
        }

        $pType = $this->filterPageType;
        if ($pType) {
            $list->filterByCollectionTypeID($pType);
        }

        $searchType = $this->searchType;
        $baseNode = $this->baseNode;

        switch ($searchType) {

            case self::SEARCH_CHILDS :
                $baseNode = Page::getCurrentPage()->getCollectionID();
                $list->filterByParentID($baseNode);
                break;

            case self::SEARCH_UNDER :
                $list->filterByParentID($baseNode);
                break;

            case self::SEARCH_ALL_CHILDS :
                $path = Page::getCurrentPage()->getCollectionPath();
                $list->filterByPath($path);
                break;

            case self::SEARCH_ALL_UNDER :
                $path = Page::getByID($baseNode)->getCollectionPath();
                $list->filterByPath($path);
                break;
        }

        $displayAliases = $this->displayAliases;
        if (!$displayAliases) {
            $list->includeAliases();
        }

        $list->filter('cvName', '', '!=');

        // Exclude pages that are excluded from all page lists :
        $db = Database::getActiveConnection();
        $columns = $db->MetaColumns(CollectionAttributeKey::getDefaultIndexedSearchTable()); 

        if (in_array('ak_exclude_page_list', $columns)) {
            $list->filter(false, '(ak_exclude_page_list = 0 or ak_exclude_page_list is null)');
        }
    }

    protected function _applySort(PageList $list)
    {
        $sortBy = $this->sortBy;
        $sortType = $this->sortType;

        $sortTypes = array(
            self::SORT_TYPE_ASC => 'ASC',
            self::SORT_TYPE_DSC => 'DESC',
        );
        $dir = $sortTypes[$sortType];

        $cnt = CollectionAttributeKey::getByHandle('gnt_ballotbox')->getController();

        $akHandle = $this->akHandle;
        $sortFields = array(
            self::SORT_BY_SCORE                 => $cnt->searchDBField(BallotBoxAttr::SCORE),
            self::SORT_BY_TOTAL_VOTES           => $cnt->searchDBField(BallotBoxAttr::TOTAL_VOTES),
            self::SORT_BY_UP_VOTES              => $cnt->searchDBField(BallotBoxAttr::UP_VOTES),
            self::SORT_BY_DOWN_VOTES            => $cnt->searchDBField(BallotBoxAttr::DOWN_VOTES),
            self::SORT_BY_TIMEFRAME_UP_VOTES    => "timeframe_up_votes",
            self::SORT_BY_TIMEFRAME_DOWN_VOTES  => "timeframe_down_votes",
            self::SORT_BY_TIMEFRAME_TOTAL_VOTES => "timeframe_down_votes",
        );

        $sortFuncs = array(
            self::SORT_BY_SITE_ORDER => array(
                'ASC'  => 'sortByDisplayOrder',
                'DESC' => 'sortByDisplayOrderDescending',
            ),
            self::SORT_BY_PUBLIC_DATE => array(
                'ASC'  => 'sortByPublicDate',
                'DESC' => 'sortByPublicDateDescending',
            ),
            self::SORT_BY_NAME => array(
                'ASC'  => 'sortByName',
                'DESC' => 'sortByNameDescending',
            ),
        );

        if (self::isTimeFrameSort($sortBy)) {
            $db = Database::getActiveConnection();

            $end = 'NOW()';
            $interval =  self::sizeToInterval($this->timeFrameSize);

            if ($this->timeFrameEnd == self::TIMEFRAME_ENDS_AT_LAST_RELEVANT) {
                $baseTs = self::sizeToBaseTs($this->timeFrameSize);

                $q = " SELECT $baseTs + INTERVAL $interval AS baseTS"
                   ." FROM btGntBallotBoxStatistics "
                   ." GROUP BY baseTS"
                   ." HAVING COUNT(statID) >= $this->relevantTimeFrame "
                   ." ORDER BY baseTS DESC";
                $r = $db->GetOne($q);

                if ($r) {
                    $end = "'$r'";
                }
            }

            $start = "$end - INTERVAL $interval";
            $subQuery = "( "
                      ." SELECT "
                      ."  cID, "
                      ."  COUNT(statID) AS timeframe_total_votes,"
                      ."  SUM( IF(vote>0,vote,0) ) AS timeframe_up_votes,"
                      ."  SUM( IF(vote<0,-vote,0)) AS timeframe_down_votes"
                      ." FROM btGntBallotBoxStatistics "
                      ." WHERE "
                      ."  cType = 'collection'"
                      ."  AND akHandle = '$akHandle' "
                      ."  AND ts >= $start "
                      ."  AND ts <= $end "
                      ." GROUP BY cID "
                      .")"; 

           // as timeframe_stats ON p1.cID = timeframe_stats.cID";

           $qb = $list->getQueryObject()->leftJoin('p', $subQuery, "timeframe_stats", "p.CID = timeframe_stats.cID");
        }


        if (array_key_exists($sortBy, $sortFields)) {
            $list->sortBy($sortFields[$sortBy], $dir);
        } else {
            $func = $sortFuncs[$sortBy][$dir];
            if (!$func) {
                Log::error(t("Up/Down Vote Lister: Invalid Sort request (sortBy=%s, dir=%s)??, please notify the developer", $sortBy, $dir));
                return;
            }
            $list->$func();
        }
    }

    private static $_viewCount = 0; // Prevent recursion

    public function view()
    {
        $this->requireAsset('gnt_ballotbox');

        $max = Package::getByHandle('gnt_vote_list')->getMaxPageLists();

        if ($max > 0 && ++self::$_viewCount > $max) {
            $this->set('pages', array());
            Log::error(t("Max Number of up/down page list exceeded ($max), stopping list to render"));

            return;
        }

        $list = $this->getPageList();

        $paginator = null;
        $pages = array();

        // Info: getPageList returns Null if the page show must be killed (if keyword is empty and empty searches forbidden)

        if (!is_null($list)) {

            if ($this->nbItemsPerPage > 0) {
                $paginator = $list->getPagination();
                $pages = $paginator->getCurrentPageResults();
            } else {
                $pages = $list->getResults();
            }
        }

        $this->set('pages', $pages);

        if ($this->displayPagination && $paginator && $paginator->getTotalPages() > 1) {
            $this->set('pagination', $paginator->renderDefaultView());
            $this->set('paginator', $paginator->renderDefaultView());
        }
    }

    public function edit()
    {
        // filtering incoherent values
        foreach (array_keys(self::getFilters()) as $filter) {
            foreach (array("Min", "Max") as $suffix) {
                $enabled = "${filter}${suffix}Enabled";
                if (!$this->$enabled) {
                    $this->set("${filter}${suffix}", '');
                }
            }
        }
    }

    public static function isTimeFrameSort($sort)
    {
        static $sorts = array(self::SORT_BY_TIMEFRAME_UP_VOTES, self::SORT_BY_TIMEFRAME_DOWN_VOTES, self::SORT_BY_TIMEFRAME_TOTAL_VOTES);

        return in_array($sort, $sorts);
    }

    public static function getFilters()
    {
        $filters = array (
            'filterUpVotes' => t('up votes between'),
            'filterDnVotes' => t('down votes between'),
            'filterTotalVotes' => t('total votes between'),
            'filterScore'      => t('score between'),
        );
        return $filters;
    }

    public static function getFilterFields()
    {
        $filters = array();
        foreach (array_keys(self::getFilters()) as $filter) {
            $filters[$filter] = uncamelcase(substr($filter,strlen('filter')));
        }
        return $filters;
    }

    public function save($args)
    {
        $args = BallotBox::prepareSaveArray($args);

        $zeroify = array('displayAliases', 'displayPagination', 'filterOnlyVotables', 'allowVoteForAll', 'allowEmptySearch');
        $intify = array('relevantTimeFrame', 'nbItemsPerPage', 'truncateSummaries', 'redirectNode', 'searchType', 'baseNode', 'sortType', 'sortBy', 'timeFrameSize', 'timeFrameEnd');


        $filters = self::getFilters();
        foreach (array_keys($filters) as $filter) {
            foreach (array('Min', 'Max') as $suffix) {
                $zeroify[] = "${filter}${suffix}Enabled";
                $intify[] = "${filter}${suffix}";
            }
        }


        foreach ($zeroify as $key) {
            if (!isset($args[$key])) {
                $args[$key] = 0;
            }
        }

        foreach ($intify as $key) {
            $args[$key] = intval($args[$key]);
        }

        parent::save($args);
    }

    public function getBallotBox($cID = null)
    {
        $box = new PageBallotBox($cID, $this->akHandle);
        $box->setController($this);

        if (is_null($cID)) {
            $this->bbox = $box;
        }

        return $box;
    }

    public function add()
    {
        $bbox = $this->getBallotBox();
        $bbox->clearImages();
    }

    protected function _getInstalledVoteTemplates()
    {
        if (!$this->_templates) {
            $btHandle  = 'gnt_vote_list';
            $pkgHandle = 'gnt_vote_list';

            $templates = array();
            $paths = array();

            $paths[] = DIR_FILES_BLOCK_TYPES."/$btHandle/vote_templates";
            $paths[] = DIR_PACKAGES."/$pkgHandle/".DIRNAME_BLOCKS."/$btHandle/vote_templates";

            $pl = PackageList::get();
            foreach ($pl->getPackages() as $pkg) {
                $handle = $pkg->getPackageHandle();
                if ($handle == $pkgHandle) {
                    continue;
                }

                $path = DIR_PACKAGES."/".$pkg->getPackageHandle()."/".DIRNAME_BLOCKS."/$btHandle/vote_templates";
                if (is_dir($path)) {
                    $paths[] = $path;
                }
            }

            foreach ($paths as $path) {
                $fileList = glob("$path/*.php");
                if (false === $fileList || !is_array($fileList)) {
                    continue;
                }

                foreach ($fileList  as $file) {
                    $base = substr(basename($file), 0, -4);
                    $name = str_replace('_', ' ', $base);
                    if (!is_array($templates[$name])) {
                        $templates[$name] = array();
                    }
                    $templates[$name][] = $file;
                }
            }
            $this->_templates = $templates;
        }

        return $this->_templates;
    }

    public function listVoteTemplates()
    {
        $list = array();
        foreach ($this->_getInstalledVoteTemplates() as $name => $files) {
            $base = substr(basename($files[0]), 0, -4);
            $list[$base] = $name;
        }

        return $list;
    }

    public function includeVoteTemplate($page, $args = array())
    {
        $akHandle = $this->akHandle;

        $ballotBox = $this->getBallotBox($page->getCollectionID());

        $upVotes = $downVotes = $score = $totalVotes = 0;

        if ($ballotBox->isInstalled()) {
            $upVotes    = $ballotBox->getUpVotes();
            $downVotes  = $ballotBox->getDownVotes();
            $score      = $ballotBox->getScore();
            $totalVotes = $ballotBox->getTotalVotes();
        } else {
            // if we have a fresh page, we might not want to allow vote:
            if ($this->allowVoteForAll == 0) {
                return;
            }
        }

        $redirectLink = $ballotBox->getRedirectLink();
        $canVote    = $ballotBox->canVote();
        $images     = $ballotBox->getImagesArray();

        $onState = $canVote ? BallotBox::STATE_ENABLED : BallotBox::STATE_DISABLED;
        $imageUp = $images[BallotBox::IMAGE_UP][$onState][$upVotes ? BallotBox::STATE_FULL : BallotBox::STATE_EMPTY];
        $imageDown = $images[BallotBox::IMAGE_DOWN][$onState][$downVotes ? BallotBox::STATE_FULL : BallotBox::STATE_EMPTY];

        $targetCID  = $page->getCollectionID();
        $bID        = $this->getBlockObject()->getBlockID();

        unset($ballotBox); // Don't export the ballot box itself

        extract($args); // allow overrides in the arguments
        include $this->getTemplatePath(); // TODO remove include statement
    }

    public function action_json_get_images()
    {
        $box = $this->getBallotBox(Page::getCurrentPage());
        $this->voteTemplate = $this->get('voteTemplate');
        print json_encode($box->getImagesArray());
        exit;
    }

    public function getTemplatePath()
    {
        $template = $this->voteTemplate;
        if (!$template) {
            $template = 'default';
        }

        $name = str_replace('_', ' ', $template);
        $templateList = $this->_getInstalledVoteTemplates();

        if (!array_key_exists($name, $templateList)) {
            $name = 'default';
        }

        return $templateList[$name][0];
    }

    public function action_json_vote()
    {
        $cID = $this->get('targetCID');
        if ('' == $cID) {
            print json_encode(array('error' => t('invalid page selection')));
            exit;
        }

        $akHandle = $this->akHandle;
        $box = new PageBallotBox($cID, $akHandle);
        $box->setController($this);
        print $box->jsONVote($this->get('vote'));
        exit;
    }

    protected static $_runningList = array();

    protected function _hashView(Block $block)
    {
        $cID   = $block->getBlockCollectionID();
        $arHandle = $block->getAreaHandle();
        $bID = $block->getBlockID();
        $key = "${arHandle}_${cID}_${bID}";

        return $key;
    }

    public function startRender(Block $block)
    {
        // must be called from the BlockView context with $this->block as parameter
        self::$_runningList[] = $this->_hashView($block);
    }

    public function endRender(Block $block)
    {
        $key = $this->_hashView($block);

        $idx = array_search($key, self::$_runningList);
        if ($idx != -1) {
            return;
        }

        unset(self::$_runningList[$idx]);
    }

    public function isRunning(Block $block)
    {
        return in_array($this->_hashView($block), self::$_runningList);
    }
}
