<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList;

defined('C5_EXECUTE') or die("Access Denied.");

use Package;
use Log;
use SinglePage;
use BlockType;
use Database;
use Events;
use Page;
use AssetList;
use Concrete\Core\Block\BlockType\Set as BlockTypeSet;
use Concrete\Core\Attribute\Type as AttributeType;
use Concrete\Package\GntVoteList\Src\VoteTarget;
use Concrete\Package\GntVoteList\Src\PageBallotBox;
use Concrete\Core\Routing\Router;
use Concrete\Core\Support\Facade\Route;

class Controller extends Package
{

    protected $pkgHandle = 'gnt_vote_list';
    protected $appVersionRequired = "5.7.5.4";
    protected $pkgVersion = "2.0.11";

    public function getPackageDescription()
    {
        return t("Allow pages to be sorted according to a user vote (up/down)");
    }

    public function getPackageName()
    {
        return t("Up Down Vote Lister");
    }

    public function install()
    {
        $pkg = parent::install();

        $bset = BlockTypeSet::getByHandle('gnt_vote');
        if (!$bset) {
            BlockTypeSet::add('gnt_vote', t('Audience Voting'), $pkg);
        }

        BlockType::installBlockTypeFromPackage("gnt_ballotbox", $pkg);
        BlockType::installBlockTypeFromPackage("gnt_vote_list", $pkg);

        $this->setMaxPageLists(100);

        // Check that the attribute Type is registered ?
        $at = AttributeType::getByHandle('gnt_ballotbox');
        if (!$at || !intval($at->getAttributeTypeID())) {
            $at = AttributeType::add('gnt_ballotbox', t('Ballot Box'), $pkg);
        }

        VoteTarget::installAttributes();

        $this->createDashboardPages($pkg);
    }

    public function createDashboardPages($pkg = null)
    {
        if (is_null($pkg)) {
            $pkg = $this;
        }

        $pages = array(
            '/dashboard/gnt_vote_list'               => array('cName' => t('Up/Down Votes'),  'cDescription' => t('Up/Down Votes Section')),
            '/dashboard/gnt_vote_list/votes'         => array('cName' => t('Page Votes'),     'cDescription' => t('Up/Down votes report')),
            '/dashboard/gnt_vote_list/stats'         => array('cName' => t('Statistics'),     'cDescription' => t('Up/Down Vote Statistics')),
            '/dashboard/gnt_vote_list/settings'      => array('cName' => t('Settings'),       'cDescription' => t("Up/Down Vote List Misc Settings")),
        );

        foreach ($pages as $path => $desc) {
            $dash = Page::getByPath($path);

            if (!$dash || $dash->isError()) {
                $dash = SinglePage::add($path, $pkg);
            }

            if (!$dash || $dash->isError()) {
                Log::warning(t("Can not install dashboard page at '%s' skipping", $path));
                continue;
            }

            $dash->update($desc);
        }
    }

    public function upgrade()
    {
        parent::upgrade();
        $this->createDashboardPages();
    }

    public function uninstall()
    {
        parent::uninstall();

        $db = Database::getActiveConnection();

        $db->Execute('DROP TABLE IF EXISTS btGntBallotBox');
        $db->Execute('DROP TABLE IF EXISTS btGntBallotBoxVoteRecord');
        $db->Execute('DROP TABLE IF EXISTS btGntBallotBoxStatistics');
        $db->Execute('DROP TABLE IF EXISTS btGntVoteList');
        $db->Execute('DROP TABLE IF EXISTS atGntBallotBox');
    }

    public function on_start()
    {
        \Events::addListener('on_page_delete', array($this, 'on_page_delete'));

        Route::register(
            Router::route(
                array('/json/clear_vote_log/{cType}/{cID}/{akHandle}', 'gnt_vote_list')), 
                '\Concrete\Package\GntVoteList\Controller\Json\ClearVoteLog::clearVotes',
                'GntVoteListClearVoteLog',
                array('cType' => '[a-zA-Z0-9_\-]+', 'cID' => '\d+', 'akHandle' => '[a-zA-Z0-9_\-]+')
        );

        Route::register(Router::route(array('/json/vote_list_images/', 'gnt_vote_list')), '\Concrete\Package\GntVoteList\Block\GntVoteList\Controller::action_json_get_images', 'GntVoteListGetImages');

        // XXX : replaces BlockType::getByHandle('gnt_vote_list')->action_json_get_images()

        Route::register(
                        Router::route(array('/ajax/render_block/{cID}/{bID}', 'gnt_vote_list')),
                        '\Concrete\Package\GntVoteList\Controller\Ajax\RenderBlock::view', 'RenderBlock',
                        array( 'cID' => '\d+', 'bID' => '\d+')
                       );

        // Register assets
        $al = AssetList::getInstance();
        $al->register('javascript', 'gnt_ballotbox', 'js/ballotbox.js', array('version' => $this->pkgVersion ), $this);
        $al->registerGroup('gnt_ballotbox', array(array( 'javascript', 'gnt_ballotbox')));
    }

    public function on_page_delete($event)
    {
        $page = $event->getPageObject();
        $box = new PageBallotBox($page->getCollectionID());
        $box->clearVoteRecords();
    }

    public function getMaxPageLists()
    {
        return intval($this->getConfig()->get('blocks.max_per_page'));
    }

    public function setMaxPageLists($max)
    {
        $this->getConfig()->save('blocks.max_per_page', intval($max));
    }
}

