<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Controller\SinglePage\Dashboard\GntVoteList;

defined('C5_EXECUTE') or die('Access Denied.');

use PageList;
use Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Package\GntVoteList\Src\PageVoteTarget;
use Concrete\Package\GntVoteList\Src\PageBallotBox;
use Page;
use Database;
use User;
use Concrete\Package\GntVoteList\Attribute\GntBallotbox\Controller as BallotBoxAttr;
use UserList;
use Log;

class Stats extends DashboardPageController
{

    protected $_targetCID         = null;
    protected $_akHandle = null;
    protected $_uID         = null;
    protected $_stats    = null;

    public function __construct(Page $c)
    {
        parent::__construct($c);
        $this->readRequest();
    }

    public function view()
    {
        // Need to load jQuery visualize
        $this->requireAsset('jquery/visualize');

        $this->set('stats',     $this->getStats());
        $this->set('targetCID', $this->_targetCID);
        $this->set('akHandle',  $this->_akHandle);
        $this->set('uID',       $this->_uID);

        $list = new PageList();

        $bList = PageVoteTarget::getBallotBoxList();

        $filter = '';
        $or = '';
        foreach (array_keys($bList) as $handle) {
            $dbField = BallotBoxAttr::getSearchDbField($handle, BallotBoxAttr::TOTAL_VOTES);
            $filter .= "$or $dbField IS NOT NULL";
            $or = " OR ";
        }
        $list->filter(false, "( $filter )");
        $list->sortByDisplayOrder();

        $bList = array_merge(array( 0 => t("All") ), $bList);
        $this->set('boxesList', $bList);

        $collections = array( 0 => t('All') );
        foreach ($list->get() as $c) {
            $collections[$c->getCollectionID()] = $c->getCollectionName();
        }
        $this->set('collections', $collections);

        $q = "SELECT DISTINCT(uID) from btGntBallotBoxStatistics";
        $userList = array( 0 => t('All') );
        $uids = Database::getActiveConnection()->GetCol($q);
        foreach ($uids as $uID) {
            $user = User::getByUserID($uID);
            if (! $user) {
                continue;
            }
            $userList[$uID] = $user->getUserName();
        }
        $this->set('userList', $userList);

        if (isset($_REQUEST['stats_cleared'])) {
            $this->set('message', t('Vote statistics had been cleared'));
        }
    }

    public function getStats()
    {

        if (is_null($this->_stats)) {
            $q = "SUM( IF( vote > 0,vote,0 ) ) AS up, "
               ."SUM( IF( vote < 0,-vote,0) ) AS down "
               ."FROM btGntBallotBoxStatistics WHERE 1=1 ";

            $dataList = array( 'up', 'down' );

            if (!is_null($this->_targetCID)) {
                $q .= "AND cID = $this->_targetCID ";
            }
            if (!is_null($this->_akHandle)) {
                $q .= "AND akHandle = '$this->_akHandle' ";
            }
            if (!is_null($this->_uID)) {
                $q .= "AND uID = $this->_uID ";
            }

            $qYear  = "SELECT MONTH(ts) as t,            $q WHERE ts > NOW() - INTERVAL 1 YEAR  GROUP BY MONTH(ts)";

            $statList = array(
                t("Last 3 hours (by 5 min)")  => array('t' => 'CONCAT(HOUR(ts)," ",FLOOR(MINUTE(ts)/5)*5)', 'i' => '3 HOUR',  'f' => '5M',    's' => '5 minute'),
                t("one day (hourly)")         => array('t' => "DATE_FORMAT(ts,'%d %H')",                    'i' => '24 HOUR', 'f' => '%d %H', 's' => '1 hour'),
                t("one month (daily)")        => array('t' => "DATE_FORMAT(ts,'%d %m')",                    'i' => '1 MONTH', 'f' => '%d %m', 's' => '1 day'),
                t("one year  (monthly)")      => array('t' => "DATE_FORMAT(ts,'%m/%y')",                    'i' => '1 YEAR',  'f' => '%m/%y', 's' => '30 day'),
            );
            $stats = array();
            $db = Database::getActiveConnection();

            $calcScore = !is_null($this->_targetCID) && !is_null($this->_akHandle) && is_null($this->_uID);
            $dataLines = array( 'total', 'up', 'down' );

            if ($calcScore) {
                $box = new PageBallotBox($this->_targetCID, $this->_akHandle);
                $finalScore = $box->getScore();
                $dataLines[] = 'score';
            }
            $this->set('dataLines', $dataLines);

            foreach ($statList as $name => $ti) {
                $now = time();
                $score = 0;
                extract($ti);
                $start = strtotime("- $i");

                # (ordering by ts to avoid issues with 't' being stringly ordered incorrectly)
                $query = "SELECT $t AS t, $q AND ts > NOW() - INTERVAL $i GROUP BY $t ORDER BY ts";
                $rs = $db->Execute($query);
                if (!$rs) {
                    Log::error(t("Up/Down Vote Lister: Can not execute [%s$query] :\n%s", $query, $db->ErrorMsg()));
                    continue;
                }

                $data = array();
                $dbData = array();
                while ($row = $rs->fetch()) {
                    $dbData[$row['t']] = $row;
                }


                while ($start <= $now) {
                    switch ($f) {
                        case '5M' :
                            $st = floor(intval(strftime('%M', $start))/5)*5;
                            $hr = intval(strftime('%H', $start));
                            $idx = sprintf('%02d %02d', $hr, $st);
                            $needle = "$hr $st";

                            break;

                        default :
                            $idx = $needle = strftime($f, $start);
                    }
                    $start = strtotime("+ $s", $start);
                    $item = array();
                    foreach ($dataList as $key) {
                        if ($this->_randomify) {
                            $item[$key] = rand(0,$this->_randomify);
                        } else {
                            $item[$key] = intval(isset($dbData[$needle][$key]) ? $dbData[$needle][$key] : 0);
                        }
                    }
                    $item['total'] = $item['up'] + $item['down'];
                    if ($calcScore) {
                        $item['score'] = $score = $score + $item['up'] - $item['down'];
                    }
                    $data[$idx] = $item;
                }
                if ($calcScore && $finalScore != $score) {
                    foreach ($data as $idx => $item) {
                        $data[$idx]['score'] += $finalScore - $score;
                    }
                }

                $stats[$name] = $data;
            }
            $this->_stats = $stats;
        }

        return $this->_stats;
    }

    public function update_stats()
    {
        $this->view();
    }

    /*
     * Non documented features, only for testing (and PRB validation), please do not use on a production
     * environment. No documentation/help will be given to end customers on how to use or troubleshoot
     * these are not supposed to be used for anything else than testing purpose
     *
     * @prb: two urls for you to play with:
     *
     * Testing the graph: /dashboard/stats/?randomify=<number>
     * Inserting random (fake) data [permanent!!]: /dashboard/stats/insert_fake_data?n=<number>
     *
     * Replace <number> by the number of data you want to show/insert
     */
    public function readRequest()
    {
        //@prb if you want to show some random data, just append ?randomify=<value> to the page url
        foreach (array( 'targetCID', 'uID', 'akHandle', 'randomify' ) as $key) {
            if (isset($_REQUEST[$key]) && $_REQUEST[$key] !== "0") {
                $oKey = "_$key";
                $this->$oKey = $_REQUEST[$key];
                $this->_stats = null;
            }
        }
    }

    public function clear_stats()
    {
        $db = Database::getActiveConnection();
        $db->Execute('DELETE FROM btGntBallotBoxStatistics');
        $this->redirect('/dashboard/gnt_vote_list/stats?stats_cleared');
    }

    public function insert_fake_data()
    {
        // @prb if you want to insert (permanently) random votes into your page, add a /fake at the end of the page url
        // Before you do that though, be sure to assign the attribute (or add the BallotBox block) to multiple pages,
        // create multiple attributes (if that is what you want to test), and multiple users etc...
        // fake datas imply that a user may have voted multiple times for the same vote (no consistency is checked on that)
        // -- anyway, that is *fake* data (use at your own risk).

        $n = 1000;
        if (isset($_REQUEST['n']) && intval($_REQUEST['n'])>0) {
            $n = intval($_REQUEST['n']);
        }

        $availableVotes = array(-1, 1);
        $startDate = strtotime("-1 year");

        $userList = new UserList();
        $userList->filterByIsActive(true);
        $userIDs = array();
        foreach($userList->get(0) as $user) {
            $userIDs[] = $user->getUserID();
        }


        $ballots = array();
        foreach (array_keys(PageVoteTarget::getBallotBoxList()) as $akHandle) {
            $pageList = new PageList();
            $pageList->filterByAttribute($akHandle, array(BallotBoxAttr::TOTAL_VOTES => null), 'IS NOT');
            foreach ($pageList->get(0) as $page) {
                $cID = $page->getCollectionID();
                $ballots[] = new PageBallotBox($cID, $akHandle);
            }
        }

        // Generate random votes
        for ($i=0; $i<$n; $i++) {
            $vote = $availableVotes[rand()%count($availableVotes)];
            $userID = $userIDs[rand()%count($userIDs)];
            $box = $ballots[rand()%count($ballots)];
            $ts = rand($startDate, time());
            $box->vote($vote, $userID, $ts);
        }

        $this->redirect('/dashboard/gnt_vote_list/stats');
    }
}

// vim: set expandtab cindent ts=4 smarttab ai:

