/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */


function BallotBoxWidget(id, imagesArray)
{
    /*
     * To Create your own widgets, just overload this object, or create a
     * compatible object, API called by the GntBallotBox class :
     *
     * object.updateVotes ( int upVotes, int downVotes, bool canVoteAgain )
     *
     * to register an object :
     * GntBallotBox.registerWidget( <?=$bID?>, myObject, <?=$akHandle?>, <?=jsONVote?> )
     */

    this.images = imagesArray;

    this.updateVotes  = function (up, down, canVoteAgain)
    { 

        var htmlID = "#" + this.id;

        $(htmlID + " span.score").text(up - down);
        $(htmlID + " span.total").text(up + down);
        $(htmlID + " span.percentUp").text((100.0*up/(up+down)).toFixed(2));
        $(htmlID + " span.percentDn").text((100.0*down/(up+down)).toFixed(2));
        $(htmlID + " span.upVotes").text(up);
        $(htmlID + " span.downVotes").text(down);

        var onState = canVoteAgain ? 'on' : 'off';
        $(htmlID + " img.upVotes"  ).attr('src', imagesArray['up'][onState][up?'full':'empty']);
        $(htmlID + " img.downVotes").attr('src', imagesArray['down'][onState][down?'full':'empty']);
    }

    /* Implementation part only : */

    this.id = id;
}

var GntBallotBox = 
{

        akHandles : {},
        blockWidgets : {},
        blocks : {},
        blocksURLs : {},
        refreshURL: '',

        // Used to vote : vote is 1 or -1, bID is your blockID, callback is a callback to be called, leave empty if you don't need it
        vote : function(vote, bID, targetCID, callback)
        {
            var data = { vote: vote };
            var url = GntBallotBox.blocksURLs[bID];

            if ('undefined' == typeof(targetCID)) {
                targetCID = CCM_CID;
            }

            var handle = GntBallotBox.getHandlefrombID( bID, targetCID )

            if ('undefined' != typeof(targetCID)) {
                data['targetCID'] = targetCID;
            }

            var o = $.getJSON ( 
                url,
                data,
                function (data)
                {
                  
                   if (!data) {
                     if (console && console.log) {
                        console.log ("Received null json response");
                     }
                     return;
                   }

                   if ('undefined' != typeof(data['warning'])) {
                       if (console && console.log) {
                           console.log(data['warning']);
                       }
                   }

                   if ("undefined" != typeof(data['error'])) {
                        if (console && console.error) {
                            console.error(data['error'])
                        } else {
                            alert(data['error']);
                        }
                        return;
                   }

                   var upVotes     = data['upVotes'];
                   var downVotes   = data['downVotes'];
                   var canVoteAgain = data['canVoteAgain'];

                   if (callback) {
                       callback(upVotes, downVotes, canVoteAgain);
                   } else {
                       if (handle) {
                           GntBallotBox.refreshHandle(handle, upVotes, downVotes, canVoteAgain, targetCID);
                       } else {
                           GntBallotBox.refreshBlock( bID, upVotes, downVotes );
                       }
                   }
                }
            );

            if ('function' == typeof(o.error)) {
                o.error(function(a, s, q) { 
                    //@PRB: for support purpose, I prefer to keep that one out of t()
                    if (console && console.error) {
                        console.error("json ERROR: " + s);
                    } else {
                        alert("json ERROR: " + s);
                    }
                    if (console && console.log) console.log(a.responseText); 
                });
            }
        },

        refreshBlock : function (bID, upVotes, downVotes, canVoteAgain, targetCID)
        {
            // refresh by javascript or by AJAX

            if ('undefined' != typeof(GntBallotBox.blockWidgets[bID])) {
                GntBallotBox.blockWidgets[bID].updateVotes(upVotes, downVotes, canVoteAgain);
                return;
            }

            if ('undefined' == typeof(targetCID)) {
                targetCID = CCM_CID;
            }

            if ( "undefined" == typeof(GntBallotBox.blocks[bID]) ) return;
            var id = GntBallotBox.blocks[bID];
            var elem = $("#" + id );
            settings = {
                data: { cID: targetCID, bID: bID },
                cache: false,
                success : function ( data, status, jq ) { elem.replaceWith( data ); },
            }
            $.ajax(GntBallotBox.refreshURL, settings);
        },

        registerBlock : function (bID, id, handle, url, targetCID)
        { 
            if (undefined == targetCID) {
                targetCID = CCM_CID;
            }

            // Prevent Double registration (in case of AJAX refresh)
            if ('undefined' != typeof (GntBallotBox.blocks[bID])) {
                return;
            }

            GntBallotBox.blocks[bID] = id;
            GntBallotBox.blocksURLs[bID] = url;
            if ( 'undefined' == typeof(GntBallotBox.akHandles[handle])) {
                GntBallotBox.akHandles[handle] = [];
            }

            if ('undefined' == typeof(GntBallotBox.akHandles[handle][targetCID])) {
                GntBallotBox.akHandles[handle][targetCID] = [];
            }

            GntBallotBox.akHandles[handle][targetCID].push(bID);
        },

        refreshHandle : function (handle, upVotes, downVotes, canVoteAgain, targetCID)
        {
           if ('undefined' == typeof(targetCID)) {
               targetCID = CCM_CID;
           }

           for (var i = 0; i < GntBallotBox.akHandles[handle][targetCID].length ; i++) {
              bID = GntBallotBox.akHandles[handle][targetCID][i];
              GntBallotBox.refreshBlock(bID, upVotes, downVotes, canVoteAgain, targetCID);
           }
        },

        getHandlefrombID : function (id, targetCID)
        {
            if ('undefined' == typeof(targetCID)) {
                targetCID = CCM_CID;
            }

            handle = false;
            $.each( 
                GntBallotBox.akHandles, function (h, v)
                {
                    if ('undefined' == typeof( v[targetCID])) {
                        return true;
                    }
                    if (-1 != $.inArray(id, v[targetCID])) {
                        handle = h;
                        return false;
                    }
                }
            );
            return handle;
        },

        registerWidget : function(bID, w, handle, url, targetCID)
        {

            if (undefined == targetCID) {
                targetCID = CCM_CID;
            }

            GntBallotBox.blockWidgets[bID] = w;
            GntBallotBox.blocksURLs[bID] = url;

            if ('undefined' == typeof(GntBallotBox.akHandles[handle])) {
                GntBallotBox.akHandles[handle] = [];
            }
            if ('undefined' == typeof(GntBallotBox.akHandles[handle][targetCID] ) ) {
                GntBallotBox.akHandles[handle][targetCID] = [];
            }

            GntBallotBox.akHandles[handle][targetCID].push(bID);
        },

        init : function (url)
        {
            // Only to be called if you plan on using the AJAX Block Refresh method
            GntBallotBox.refreshURL = url;
        }

}
