<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Package\GntVoteList\Attribute\GntBallotbox\Controller;
use Database;
use Package;
use User;
use File;
use View;
use Page;
use Log;
use Session;

/*
 * The real thing, handle the attribute and record votes
 */

class BallotBox
{

    private $_controller = null;    // Controller object to save and retrieve values from (to be used for controls)
    private $_voteTarget = null;    // Target object the vote is upon (Page ...) VoteTarget object
    private $_uID = null;           // Current user that will vote (if any)
    private $_records;              // default storage for ballotbox behaviour controls (not much used though)

    public function __construct(VoteTarget $vtarget)
    {
        $this->_voteTarget = $vtarget;

        // _records store all behaviour related data, keys are used to setup
        // behaviour from a target object
        $this->_records = array(
            'voteEnabled'          => 1,
            'anonymousCanVote'     => 0,
            'allowVoteChange'      => 1,
            'voteOpening'          => null,
            'voteOpening_activate' => 0,
            'voteClosing'          => null,
            'voteClosing_activate' => 1,
            'recordStats'          => 1,
            'redirectType'         => self::REDIRECT_TO_NOWHERE,
            'redirectUrl'          => null,
            'redirectNode'         => null,
        );
    }

    //
    // Clear records for a particular uID, the ballotBox can be pointed to a null handle to clear them all
    //
    public function clearVoteRecords($uID = null)
    {
        $db = Database::getActiveConnection();
        list($cID, $cType, $akHandle) = $this->_getVoteCard();

        $q = "DELETE FROM btGntBallotBoxVoteRecord WHERE cType = ?";
        $args = array($cType);

        if (!is_null($cID)) {
            $q .= " AND cID = ?";
            $args[] = $cID;
        }

        if (!is_null($akHandle)) {
            $q .= " AND akHandle = ?";
            $args[] = $akHandle;
        }

        if (!is_null($uID)) {
            $q = "$q AND uID = ?";
            $args[] = $uID;
        }

        $db->Execute($q, $args);
    }

    public static function clearAllVoteRecords()
    {
        $db = Database::getActiveConnection();
        $db->Execute('DELETE FROM btGntBallotBoxVoteRecord');
    }

    public function getRecordsCount()
    {
        $db = Database::getActiveConnection();
        $args = $this->_getVoteCard();

        return $db->GetOne('SELECT COUNT(voteID) FROM btGntBallotBoxVoteRecord WHERE cID = ? AND  cType = ? AND akHandle = ?', $args);
    }

    public function setVoteTargetById($id, $akHandle = 'gnt_ballotbox', $type = 'collection')
    {
        $this->_voteTargetget = VoteTarget::factory($id, $type, $akHandle);
    }

    public function setVoteTarget(VoteTarget $target)
    {
        $this->_voteTarget = $target;
    }
    public function getVoteTarget()
    {
        return $this->_voteTarget;
    }

    public function getUserVote()
    {
        list($cID, $cType, $akHandle) = $this->_getVoteCard();

        if ($this->isAnonymous()) {
            $sessionKey = "gnt_vote_list::${akHandle}/${cType}.${cID}";
            if (!Session::has($sessionKey)) {
                return false;
            }
            return Session::get($sessionKey);
        }

        $db = Database::getActiveConnection();
        $uID = $this->getUserID();

        $r = $db->GetOne(
            "SELECT vote FROM btGntBallotBoxVoteRecord WHERE cID = ? AND uID = ? AND cType = ? AND akHandle = ?", array( $cID, $uID, $cType, $akHandle )
        );

        if (is_null($r)) {
            return false;
        }

        return $r;
    }
    // accessors :
    public function voteEnabled()
    {
        return $this->get('voteEnabled') != 0;
    }

    public function anonymousCanVote()
    {
        return $this->get('anonymousCanVote') != 0;
    }

    public function recordStats()
    {
        return $this->get('recordStats') != 0;
    }

    public function hasVoteStart()
    {
        return $this->get('voteOpening_activate') != 0;
    }

    public function hasVoteEnd()
    {
        return $this->get('voteClosing_activate') != 0;
    }

    public function voteStarted()
    {
        return !$this->hasVoteStart() || $this->voteOpening() <=  new \DateTime();
    }
    public function voteEnded()
    {
        return $this->hasVoteEnd() && $this->voteClosing() <=  new \DateTime();
    }

    public function voteOpening()
    {
        return new \DateTime($this->get('voteOpening'));
    }

    public function voteClosing()
    {
        return new \DateTime($this->get('voteClosing'));
    }

    public function redirectNode()
    {
        return $this->get('redirectNode');
    }

    public function redirectType()
    {
        return $this->get('redirectType');
    }

    public function redirectUrl()
    {
        return $this->get('redirectUrl');
    }

    public function voteOpened()
    {
        return $this->voteStarted() && !$this->voteEnded();
    }

    public function allowVoteChange()
    {
        return $this->get('allowVoteChange') != 0;
    }

    public function isAnonymous()
    {
        return $this->getUserID() == null;
    }

    public function getUpVotes()
    {
        return $this->_voteTarget->getUpVotes();
    }

    public function getDownVotes()
    {
        return $this->_voteTarget->getDownVotes();
    }

    public function getScore()
    {
        return $this->_voteTarget->getScore();
    }

    public function getTotalVotes()
    {
        return $this->_voteTarget->getTotalVotes();
    }

    public function isVotable()
    {
        if (!$this->voteEnabled()) {
            return false;
        }

        if (!$this->anonymousCanVote() && $this->isAnonymous()) {
            return false;
        }

        if (!$this->voteOpened()) {
            return false;
        }

        return true;
    }

    public function setUserID($uID)
    {
        $this->_uID = $uID;
    }

    public function getUserID()
    {
        if (is_null($this->_uID)) {
            $user = new User();
            if ($user->checkLogin()) {
                $this->_uID = $user->getUserID();
            }
        }

        return $this->_uID;
    }

    // Meta-Composition : add BallotBox like controls to an object :
    public function setController($controller)
    {
        $this->_controller = $controller;
    }

    public function get($key)
    {
        if (!$this->_controller || $this == $this->_controller) {
            return $this->_records[$key];
        }
        $val = $this->_controller->$key;
        return $val;
    }

    public function set($key, $val)
    {

        if (!$this->_controller || $this == $this->_controller) {
            $this->_records[$key] = $val;
            return;
        }
        $this->_controller->$key = $val;
    }

    public function prepareSaveArray(array $args)
    {
        // checkboxes are just omitted if not checked :
        $checkboxes = array('voteEnabled', 'anonymousCanVote', 'allowVoteChange', 'recordStats', 'voteClosing_activate', 'voteOpening_activate');
        foreach ($checkboxes as $key) {
            if (!isset($args[$key])) {
                $args[$key] = 0;
            }
        }

        $intify = array_keys(self::getAllImageHandles('image'));
        $intify[] = 'redirectNode';
        foreach ($intify as $key) {
            $args[$key] = intval($args[$key]);
        }

        $dt   = \Core::make('helper/form/date_time');
        $date = \Core::make('helper/date');

        if ($args['voteClosing_activate']) {
            $args['voteClosing'] = $date->getSystemDateTime($dt->translate('voteClosing'));
        }

        if ($args['voteOpening_activate']) {
            $args['voteOpening'] = $date->getSystemDateTime($dt->translate('voteOpening'));
        }

        return $args;
    }

    public function canVote()
    {
        if (!$this->isVotable()) {
            return false;
        }

        if ($this->didVote() && !$this->allowVoteChange()) {
            return false;
        }

        return true;
    }

    public function didVote()
    {
        return false !== $this->getUserVote();
    }

    protected function _getVoteCard()
    {
        if (!$this->_voteTarget) {
            return array(null, null, null);
        }

        $cID      = $this->_voteTarget->getID();
        $cType    = $this->_voteTarget->getType();
        $akHandle = $this->_voteTarget->getAkHandle();

        return array( $cID, $cType, $akHandle );
    }

    protected function _saveUserVote($vote, $userOrUserID=null)
    {

        list($cID, $cType, $akHandle) = $this->_getVoteCard();

        if ($this->isAnonymous()) {
            Session::set("gnt_vote_list::${akHandle}/${cType}.${cID}", $vote);
        } else {

            $uID = $userOrUserID;
            if (is_object($userOrUserID)) {
                $uID = $userOrUserID->getUserID();
            } else if (is_null($userOrUserID)) {
                $user = new User();
                $uID = $user->getUserID();
            }

            $db = Database::getActiveConnection();

            $record = array('uID' => $uID, 'cID' => $cID, 'vote' => $vote, 'cType' => $cType, 'akHandle' => $akHandle);

            $voteID = $db->GetOne(
                "SELECT voteID FROM btGntBallotBoxVoteRecord WHERE cID = ? AND uID = ? AND cType = ? AND akHandle = ?", array($cID, $uID, $cType, $akHandle)
            );

            if (!is_null($voteID) && $voteID !== false) {
                $db->update("btGntBallotBoxVoteRecord", $record, array("voteID" => $voteID));
                if ($this->recordStats()) {
                    $statID = $db->GetOne(
                        "SELECT statID FROM btGntBallotBoxStatistics WHERE cID = ? AND uID = ? AND cType = ? AND akHandle = ? ORDER BY ts DESC", array($cID, $uID, $cType, $akHandle)
                    );

                    if (!is_null($statID)) {
                        $db->update("btGntBallotBoxStatistics", $record, array("statID" => $statID));
                    }
                }
            } else {
                $db->insert("btGntBallotBoxVoteRecord", $record);
            }
        }
    }

    // do the actual vote in the attribute :
    protected function _addVotes($up, $down)
    {
        $this->_voteTarget->addVotes($up, $down);
    }

    public function reset()
    {
        if (! $this->_voteTarget->isValid()) {
            return;
        }
        $this->_voteTarget->reset();
    }

    const VOTE_UP   = 1;
    const VOTE_DOWN = -1;

    protected function _vote($vote)
    {
        $this->_addVotes($vote == 1 ? 1 : 0, $vote == -1 ? 1 : 0);
    }

    protected function _undoVote($vote)
    {
        $this->_addVotes($vote == 1 ? -1 : 0, $vote == -1 ? -1 : 0);
    }

    public function vote($vote, $userOrUserID=null, $ts=null)
    {
        $this->_vote($vote);
        $this->_saveUserVote($vote, $userOrUserID);
        $this->_recordStats($vote, $userOrUserID, $ts);
    }

    protected function _recordStats($vote, $userOrUserID=null, $ts=null)
    {
        if (!$this->recordStats()) {
            return;
        }

        list($cID, $cType, $akHandle) = $this->_getVoteCard();
        $uID = null;

        if (!$this->isAnonymous()) {

            $uID = $userOrUserID;
            if (is_object($userOrUserID)){
                $uID = $userOrUserID->getUserID();
            } else if (is_null($userOrUserID)){
                $user = new User();
                $uID = $user->getUserID();
            }
        }

        if (!is_integer($ts)) {
            $ts = time();
        }
        $db = Database::getActiveConnection();
        $record = array('uID' => $uID, 'vote' => $vote, 'cID' => $cID, 'cType' => $cType, 'akHandle' => $akHandle, 'ts' => strftime("%F %T", $ts));
        $db->insert("btGntBallotBoxStatistics", $record);
    }

    public function changeVote($vote)
    {
        $lastVote = $this->getUserVote();
        if ($lastVote == $vote) {
            return false;
        }

        $this->_undoVote($lastVote);
        $this->_vote($vote);
        $this->_saveUserVote($vote);

        return true;
    }

    public function isValid()
    {
        return !is_null($this->_voteTarget) && $this->_voteTarget->isValid();
    }

    // tool related function :
    public function jsONVote($vote)
    {
        if (!$this->canVote()) {
            return json_encode(array('error' => t('Can not vote!')));
        }

        if ('' == $vote) {
            return json_encode(array('error' => t('Invalid vote!')));
        }

        if (!$this->_voteTarget->isValid()) {
            return json_encode(array('error' => t('page not found')));
        }

        $r = array();

        if ($this->didVote()) {
            if (!$this->changeVote($vote)) {
                $r['warning'] = t("Last vote is identical, not changing");
            }
        } else {
            $this->vote($vote);
        }

        $r['upVotes']      = $this->getUpVotes();
        $r['downVotes']    = $this->getDownVotes();
        $r['canVoteAgain'] = $this->canVote();

        return json_encode($r);
    }

    protected function _element($element, $vars = null)
    {
        if (is_null($vars)) {
            $vars = array();
        }

        $args = array();

        foreach (array_keys($this->_records) as $key) {
            $args[$key] = isset($vars[$key]) ? $vars[$key] : $this->get($key);
        }

        $args['bbox'] = $this;
        View::element($element, $args, 'gnt_vote_list');
    }

    public function editForm($vars = null)
    {
        $this->_element('ballotbox_edit', $vars);
    }

    public function editImages($vars = null)
    {
        $this->_element('ballotbox_images', $vars);
    }

    public function isInstalled()
    {
        return $this->_voteTarget->isInstalled();
    }

    public function install()
    {
        if (!$this->isInstalled()) {
            $this->_voteTarget->initVotes();
        }
    }

    public function computeMessage($message)
    {
        $up    = $this->getUpVotes();
        $down  = $this->getDownVotes();
        $total = $this->getTotalVotes();
        $score = $this->getScore();

        $totalDiv = $total ?  $total : 1;

        $percentUp = sprintf("%.2f", (100.0*$up/($totalDiv)));
        $percentDn = sprintf("%.2f", (100.0*$down/($totalDiv)));

        $target = $this->getVoteTarget();

        $message = $target->computeMessage($message);

        $replace = array(
            '/%UP%/'       => "<span class=\"upVotes\">$up</span>",
            '/%DOWN%/'     => "<span class=\"downVotes\">$down</span>",
            '/%TOTAL%/'    => "<span class=\"total\">$total</span>",
            '/%SCORE%/'    => "<span class=\"score\">$score</span>",
            '/%PERCENTUP%/' => "<span class=\"percentUp\">$percentUp</span>",
            '/%PERCENTDN%/' => "<span class=\"percentDn\">$percentDn</span>",
        );

        $message = preg_replace(array_keys($replace), array_values($replace), $message);

        return $message;
    }

    /* image interface */

    protected $_defaultImgs = null;

    protected function _computeImageDefaults()
    {
        if (!is_null($this->_defaultImgs)) {
            return;
        }

        $handles = self::getAllImageHandles('image');
        $template = $this->_controller->getTemplatePath();

        foreach ($handles as $handle => $desc) {
            list($dir, $onState, $state) = $desc;
            $this->_defaultImgs[$dir][$onState][$state] = -1;
        }

        $fh = \Core::make('helper/file');

        $content = $fh->getContents($template);

        $dirs = array_keys(self::getVotingDirections());

        foreach (preg_split('/\n/', $content) as $line) {
            if ('#' != $line[0]) {
                continue;
            }

            $matches = array();
            if (preg_match_all('/^# IMAGE_(UP|DOWN)_(ON|OFF)\s+([^[:space:]]+)\s+([^[:space:]]+)(\s+)?([^[:space:]]+)?$/', $line, $matches)) {
                $type     = strtolower($matches[1][0]);
                $onState  = strtolower($matches[2][0]);

                if (!in_array($type, $dirs)) {
                    continue;
                }

                $this->_defaultImgs[$type][$onState] = array(
                    self::STATE_FULL     => $matches[3][0],
                    self::STATE_EMPTY    => $matches[4][0],
                );
            }
        }
    }

    const IMAGE_UP   = 'up';
    const IMAGE_DOWN = 'down';

    public static function getVotingDirections()
    {
        return array(self::IMAGE_UP => t('Up'), self::IMAGE_DOWN => t('Down'));
    }

    const STATE_EMPTY    = 'empty';
    const STATE_FULL     = 'full';

    const STATE_ENABLED  = 'on';
    const STATE_DISABLED = 'off';

    public static function getStates()
    {
        return array(self::STATE_EMPTY => t('empty'), self::STATE_FULL => t('full'));
    }

    public static function getActivationStates()
    {
        return array(self::STATE_ENABLED => t('voting on'), self::STATE_DISABLED => t('voting off'));
    }

    public static function getAllImageHandles($prefix='img')
    {
        $dirs     = array_keys(self::getVotingDirections());
        $onStates = array_keys(self::getActivationStates());
        $states   = array_keys(self::getStates());

        $handles =  array();

        foreach ($dirs as $dir) {
            foreach ($onStates as $onState) {
                foreach ($states as $state) {
                    $handles["${prefix}_${dir}_${onState}_${state}"] = array($dir, $onState, $state);
                }
            }
        }
        return $handles;
    }

    public static function clearImagesRecord(array &$args)
    {
        foreach (array_keys(self::getAllImageHandles('image')) as $handle) {
            $args[$handle] = -1;
        }
        return $args;
    }

    public function clearImages()
    {
        foreach (array_keys(self::getAllImageHandles('image')) as $handle) {
            $this->set($handle, -1);
        }
        return $args;
    }

    public function getImage($dir, $activate, $state)
    {
        $this->_computeImageDefaults();
        $name = "image_${dir}_${activate}_${state}";

        $image = $this->get($name);

        if (!is_null($image)) {
            $file = File::getByID($image);
            if ($file && !$file->error) {
                return $file->getRelativePath();
            }
        }

        return $this->getDefaultImage($dir, $activate, $state);
    }

    public function getDefaultImage($dir, $activate, $state)
    {
        $this->_computeImageDefaults();
        $image = "no-default.png";
        if ($this->_defaultImgs[$dir][$activate][$state] != -1) {
            $image = $this->_defaultImgs[$dir][$activate][$state];
        }

        return $this->getImagePath($image);
    }

    public function getImagePath($image)
    {
        if ('/' == $image[0]) {
            return $image;
        }

        $pkg = Package::getByHandle('gnt_vote_list');
        $uh  = \Core::make('helper/concrete/urls');
        $path = $uh->getPackageURL($pkg)."/images";

        return "$path/$image";
    }

    public function getImagesArray()
    {
        $images = array();

        # TODO: refactor to have a common prefix image_ or img_ everywhere in getAllImageHandles()
        #
        foreach (self::getAllImageHandles() as $img => $desc) {
            list($dir, $onState, $state) = $desc;
            $var = camelcase($img);
            $var[0] = strtolower($var[0]);


            $images[$dir][$onState][$state] = $this->getImage($dir, $onState, $state);
        }
        return $images;
    }

    const REDIRECT_TO_NOWHERE  = 0;
    const REDIRECT_TO_LOGIN    = 1;
    const REDIRECT_TO_REGISTER = 2;
    const REDIRECT_TO_PAGE     = 3;
    const REDIRECT_TO_URL      = 4;

    public static function getRedirectTypes()
    {
        $ret = array(
            self::REDIRECT_TO_NOWHERE  => t('Nowhere'),
            self::REDIRECT_TO_LOGIN    => t('Login page'),
            self::REDIRECT_TO_REGISTER => t('Register Page'),
            self::REDIRECT_TO_PAGE     => t('Selected Page'),
            self::REDIRECT_TO_URL      => t('URL'),
        );

        if (ENABLE_REGISTRATION != 1) {
            unset($ret[self::REDIRECT_TO_REGISTER]);
        }

        return $ret;
    }

    public function getRedirectLink()
    {
        if (!$this->voteEnabled() || !$this->isAnonymous() || $this->anonymousCanVote()) {
            return;
        }

        $rType = $this->redirectType();

        switch ($rType) {

            case self::REDIRECT_TO_NOWHERE:  return;
            case self::REDIRECT_TO_LOGIN:    return View::url('/login');
            case self::REDIRECT_TO_REGISTER: return View::url('/register');
            case self::REDIRECT_TO_URL:      return $this->redirectUrl();

            case self::REDIRECT_TO_PAGE :

                $node = $this->redirectNode();
                if (!$node) {
                    return;
                }

                $nav = \Core::make('helper/navigation');
                $page = Page::getByID($node);

                if (!$page || $page->isError()) {
                    return;
                }

                return $nav->getLinkToCollection($page);
        }
    }
}

// vim: set expandtab cindent ts=4 smarttab ai:

