<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Page;

class PageBallotBox extends BallotBox
{

    public function __construct($cID = null, $akHandle = null)
    {
        $target = new PageVoteTarget($cID, $akHandle);
        parent::__construct($target);
    }

    public function setPageTarget(Page $page)
    {
        $this->getVoteTarget()->setID($page->getCollectionID());
    }
    public function setPageTargetId($cID)
    {
        $this->getVoteTarget()->setID($cID);
    }

    public function getTargetCID()
    {
        return $this->getVoteTarget()->getID();
    }

    protected function _getPage()
    {
        return Page::getByID($this->getVoteTarget()->getID());
    }

    # compute page descriptions and truncate them to match patterns
    public function computePageDescs($message)
    {
        $th = \Core::make('helper/text');

        $page = $this->_getPage();
        $desc = $page->getCollectionDescription();

        if (preg_match('/%PAGEDESC_\d+%/', $message)) {
            $matches = array();
            preg_match_all('/%PAGEDESC_(\d+)%/', $message, $matches);
            $truncateList = array_unique($matches[1]);
            foreach ($truncateList as $sz) {
                $message = preg_replace("/%PAGEDESC_$sz%/", $th->shorten($desc, intval($sz)), $message);
            }
        }

        return $message;
    }

    public function getPageThumbnail()
    {
        $page = $this->_getPage();
        $thumb = $page->getAttribute('thumbnail');

        if (!is_null($thumb) && $thumb !== false) {
            return $thumb;
        }

        try {
            // Sooooo hacky, brrrr
            // Using the BlogEntry method (5.6 way, for compatibility only)
            $ts = $page->getBlocks('Thumbnail Image');
            $imgBox = null;
            if (is_object($ts[0])) {
                $imgBox = $ts[0]->getInstance();
            }
            if (method_exists($imgBox, 'getFileObject')) {
                return $imgBox->getFileObject();
            }
        } catch (Exception $e) {
        }

        return $thumb;
    }

    public function computeThumbs($message)
    {
        if (!preg_match('/%THUMB(_\d+x\d+)?%/', $message)) {
            return $message;
        }

        $thumb = $this->getPageThumbnail();
        if (! $thumb) {
            return preg_replace('/%THUMB(_\d+_\d+)?%/', '', $message);
        }

        $sizes = array();
        if (preg_match('/%THUMB_\d+x\d+%/', $message)) {
            $matches = array();
            preg_match_all('/%THUMB_(\d+)x(\d+)%/', $message, $matches);
            $width  = $matches[1];
            $height = $matches[2];
            foreach (array_keys($width) as $i) {
                $w = intval($width[$i]);
                $h = intval($height[$i]);
                if (220 == $w && 220 == $h) {
                    continue;
                }
                $sizes[] = array( $w, $h );
            }
            $sizes = array_unique($sizes);
        }

        // TODO XXX FIXME should move to the new image process when it is defined
        // see github issue: https://github.com/concrete5/concrete5-5.7.0/issues/1152
        // and forum post: http://www.concrete5.org/community/forums/5-7-discussion/image-helper/

        $imgHelper = \Core::make('helper/image');

        $thumbnail = $imgHelper->getThumbnail($thumb, 220, 220, false);

        $w = $thumbnail->width;
        $h = $thumbnail->height;
        $img = $thumbnail->src;

        # replace default image :
        $imgTag = "<img src=\"$img\" width=\"220\" height=\"220\" alt=\"".t("Page thumbnail").'" />';
        $message = preg_replace('/%THUMB(_220x220)?%/', $imgTag, $message);

        foreach ($sizes as $size) {
            list($w, $h) = $size;
            $thumbnail = $imgHelper->getThumbnail($thumb, $w, $h, false);

            $width = $thumbnail->width;
            $height = $thumbnail->height;
            $img = $thumbnail->src;

            $imgTag = "<img src=\"$img\" width=\"$width\" height=\"$height\" alt=\"".t("Page thumbnail").'" />';
            $message = preg_replace("/%THUMB_${w}x${h}%/", $imgTag, $message);
        }

        return $message;
    }

    # Compute the message to include page specific patterns
    public function computeMessage($message)
    {
        $message = parent::computeMessage($message);
        $page = $this->_getPage();

        $replace = array(
            '/%PAGENAME%/' => $page->getCollectionName(),
            '/%PAGEDESC%/' => $page->getCollectionDescription(),
        );

        $message = preg_replace(array_keys($replace), array_values($replace), $message);
        $message = $this->computePageDescs($message);
        $message = $this->computeThumbs($message);

        return $message;
    }
}

// vim: set expandtab cindent ts=4 smarttab ai:

