<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 *          -----------------------------------------------------              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Page;
use Concrete\Core\Attribute\Key\CollectionKey as CollectionAttributeKey;
use Concrete\Package\GntVoteList\Attribute\GntBallotbox\Controller as BallotAttrController;

/*
 * interface for pages to be votable
 */

class PageVoteTarget extends VoteTarget
{

    protected static $_targetCID = null;

    public static function setTargetCID($targetCID)
    {
        self::$_targetCID = $targetCID;
    }
    public static function resetTargetCID()
    {
        self::$_targetCID = null;
    }

    public static function getBallotBoxList()
    {
        return parent::getBallotBoxListForType('collection');
    }

    public function __construct($id, $akHandle)
    {
        parent::__construct($id, $akHandle, 'collection');
    }

    protected function _getAttributeValue($create = true)
    {
        if (! $this->isValid()) {
            return;
        }

        $page = $this->_getPage();
        $akHandle = $this->getAkHandle();

        // ensure the attribute type is correctly bound to the collection :
        self::createAttribute($akHandle);

        $ak = CollectionAttributeKey::getByHandle($akHandle);

        $val = $page->getAttribute($akHandle);
        if (!$val && $create) {
            $val = BallotAttrController::newValue();
        }

        return $val;
    }

    protected function _setAttributeValue($val)
    {
        $this->_getPage()->setAttribute($this->getAkHandle(), $val);
    }

    protected function _getPage()
    {
        $id = $this->_getID();
        if (is_null($id) and !is_null(self::$_targetCID)) {
            $id = self::$_targetCID;
        }

        if (!is_null($id)) {
            $page = Page::getByID($id);
        }
        if (is_null($page)) {
            $page = Page::getCurrentPage();
        }

        return $page;
    }

    public function getID()
    {
        $page = $this->_getPage();

        return is_null($page) ? null : $page->getCollectionID();
    }

    public function isValid()
    {
        return $this->_getPage() != null && !is_null($this->getAkHandle());
    }
}

// vim: set expandtab cindent ts=4 smarttab ai:

