<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntVoteList\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Attribute\Type              as AttributeType;
use Concrete\Core\Attribute\Key\Key          as AttributeKey;
use Concrete\Core\Attribute\Key\CollectionKey as CollectionAttributeKey;
use Concrete\Core\Attribute\Key\Category      as AttributeKeyCategory;
use Log;
use Package;

/*
 * Target interface of the Vote
 */

class VoteTarget
{
    protected $_id;
    protected $_type;
    protected $_akHandle;

    public function __construct($id = null, $akHandle = null, $type = null)
    {
        $this->_id = $id;
        $this->_type = $type;
        $this->_akHandle = $akHandle;
    }

    protected function _getID()
    {
        return $this->_id;
    }
    public function getID()
    {
        return $this->_getID();
    }
    public function setID($id)
    {
        $this->_id = $id;
    }
    public function getCollection()
    {
        return;
    }

    public function getType()
    {
        return $this->_type;
    }
    public function getAkHandle()
    {
        return $this->_akHandle;
    }

    public function setAkHandle($handle)
    {
        $this->_akHandle = $handle;
    }

    protected function _getAttributeValue($create = true)
    {
        return;
    }
    protected function _setAttributeValue($val)
    {
    }

    protected static $_targetTypes = array(
        'collection' => 'PageVoteTarget',
    );

    public static function factory($type, $id, $handle)
    {
        $class = self::$_targetTypes[$type];
        if ($class) {
            return new $class($id, $handle);
        }
    }

    public static function registerVoteTargetType($type, $class)
    {
        self::$_targetTypes[$type] = $class;
    }

    public static function installAttributes($name = null)
    {
        if (is_null($name)) {
            $name = t("Vote List Ballot Box");
        }

        $at = AttributeType::getByHandle('gnt_ballotbox');
        if (!$at) {
            Log::error(t('Can not find attribute %s, please re-install addon', 'gnt_ballotbox'));

            return;
        }

        foreach (array_keys(self::$_targetTypes) as $target) {
            $akc = AttributeKeyCategory::getByHandle($target);
            if (!$akc) {
                Log::error(t("VoteList can not associate attribute to non existing %s category", $target));
                continue; // sanity check
            }

            if (!$at->isAssociatedWithCategory($akc)) {
                $akc->associateAttributeKeyType($at);
            }
        }

        // Make sure collections have at least one attribute available
        self::createAttribute('gnt_ballotbox', t("Vote List Ballot Box"));
    }

    // Ensure the attribute is created (putting a default name if nonexisting)
    public static function createAttribute($akHandle, $name = null)
    {
        $key = CollectionAttributeKey::getByHandle($akHandle);

        if ($key && intval($key->getAttributeKeyID())) {
            return;
        }

        if (is_null($name)) {
            $name = t("Recovered Ballot Box");
        }

        $desc = array( 'akHandle' => $akHandle, 'akName' => $name, 'akIsSearchable' => false );

        $key = CollectionAttributeKey::add('gnt_ballotbox', $desc, Package::getByHandle('gnt_vote_list'));
    }

    public function isValid()
    {
        return true;
    }

    public static function getBallotBoxListForType($type)
    {
        $akList = array();
        $atID = AttributeType::getByHandle('gnt_ballotbox')->getAttributeTypeID();
        $list = AttributeKey::getList($type, array( 'atID' => $atID ));
        foreach ($list as $ak) {
            $akList[$ak->getAttributeKeyHandle()] = $ak->getAttributeKeyName();
        }

        return $akList;
    }

    public function addVotes($up, $down)
    {
        if (!$this->isValid()) {
            return false;
        }
        $av = $this->_getAttributeValue();
        $av->addUpVotes($up);
        $av->addDownVotes($down);
        $this->_setAttributeValue($av);

        return true;
    }

    public function initVotes()
    {
        if (!$this->isValid()) {
            return false;
        }
        $av = $this->_getAttributeValue();
        $this->_setAttributeValue($av);
    }

    public function reset()
    {
        if (!$this->isValid()) {
            return false;
        }
        $av = $this->_getAttributeValue();
        $av->setUpVotes(0);
        $av->setDownVotes(0);
        $this->_setAttributeValue($av);
    }

    public function getUpVotes()
    {
        return !$this->isValid() ? false : $this->_getAttributeValue()->getUpVotes();
    }
    public function getDownVotes()
    {
        return !$this->isValid() ? false : $this->_getAttributeValue()->getDownVotes();
    }
    public function getScore()
    {
        return !$this->isValid() ? false : $this->_getAttributeValue()->getScore();
    }
    public function getTotalVotes()
    {
        return !$this->isValid() ? false : $this->_getAttributeValue()->getTotalVotes();
    }
    public function isInstalled()
    {
        return $this->_getAttributeValue(false) != null;
    }

    // this method is used to compute the target related message patterns
    public function computeMessage($message)
    {
        return $message;
    }
}

// vim: set expandtab cindent ts=4 smarttab ai:

